import firebase_admin
from firebase_admin import credentials
from firebase_admin import firestore
from datetime import datetime
from pytz import timezone
import pytz

# Use the application default credentials
# cred = credentials.ApplicationDefault()
# firebase_admin.initialize_app(cred, {
#     'projectId': os.environ.get("GCP_PROJECT_ID"),
# })
# firebase_admin.initialize_app(cred, {
#     'projectId': "data8x-scratch",
# })
cred = credentials.ApplicationDefault()
firebase_admin.initialize_app(cred, {
    'projectId': "data8x-scratch",
    'storageBucket': 'data8x-scratch.appspot.com/submissions'
})
db = firestore.client()
date_format = "%Y-%m-%d %H:%M:%S %Z"
date = datetime.now(tz=pytz.utc)
date = date.astimezone(timezone('US/Pacific'))
data = {
    'user': '543',
    'grade': 1.0,
    'section': '2',
    'lab': 'lab01',
    'date': date.strftime(date_format)
}
doc = db.collection("otter-test").add(data)[1]
doc_ref = db.collection("otter-test").document(f'{doc.id}')
doc = doc_ref.get()
print(doc.to_dict())



import otter_service.access_sops_keys as ask
import pytest
import os


@pytest.fixture
def setup():
    os.environ["LTI_CONSUMER_KEY"] = "TEST_ENV_KEY"
    yield
    if 'LTI_CONSUMER_KEY' in os.environ:
        del os.environ['LTI_CONSUMER_KEY']


def test_get_sops():
    secrets_path = os.path.join(os.path.dirname(__file__), "test_files/test_key.yaml")
    sops_path = "sops"
    key = ask.get_via_sops(None, "LTI_CONSUMER_KEY", sops_path=sops_path, secrets_file=secrets_path)
    assert "test_lti_key" in key


def test_get_via_env(setup):
    key = ask.get_via_env(None, "LTI_CONSUMER_KEY")
    assert "TEST_ENV_KEY" in key


def test_get(setup):
    key = ask.get(None, "LTI_CONSUMER_KEY")
    assert "TEST_ENV_KEY" in key
    del os.environ['LTI_CONSUMER_KEY']
    try:
        key = ask.get("LTI_CONSUMER_KEY")  # this should raise Exception
        assert False
    except Exception:
        assert True



import asyncio
import async_timeout
import requests
import tarfile
import os
import glob
from otter_service import access_sops_keys
import shutil
import otter_service.util as util


def download_autograder_materials(course, sops_path, secrets_file, save_path=None):
    """
    This function downloads the appropriate archive of autograder materials be used for testing. The archive
    must be on the main branch(but it will also try the master branch if main fails)

    :param course: key to git access token in secrets file
    :param sops_path: path to sops executable
    :param secrets_file: path to secrets file
    :param save_path: Where to save the archive -- for testing it is "." for normal it will /tmp
    """
    branch = "main"
    git_access_token = access_sops_keys.get(course, "github_access_token", sops_path=sops_path, secrets_file=secrets_file)
    autograder_materials_repo = access_sops_keys.get(course, "autograder_repo", sops_path=sops_path, secrets_file=secrets_file)
    materials_url = f"https://{git_access_token}:@{autograder_materials_repo}/archive/{branch}.tar.gz"
        
    download_path = "/tmp/materials.tar.gz"
    if save_path is None:
        save_path = "."
        download_path = "./materials.tar.gz"
    r = requests.get(materials_url, stream=True)
    if r.status_code != 200:
        branch = "master"
        materials_url = f"https://{git_access_token}:@{autograder_materials_repo}/archive/{branch}.tar.gz"    
        r = requests.get(materials_url, stream=True)

    if r.status_code == 200:
        with open(download_path, 'wb') as f:
            f.write(r.raw.read())
        file = tarfile.open(download_path)
        file.extractall(save_path)
        file.close()
        url_parts = materials_url.split("/")
        branch = url_parts[-1].split(".")[0]
        file_name = autograder_materials_repo.split("/")[-1]
        extracted_path = f"{save_path}/{file_name}-{branch}"
        storage_path = f"{save_path}/{file_name}"
        if os.path.isdir(storage_path):
            shutil.rmtree(storage_path)
        os.rename(extracted_path, storage_path)
        os.remove(download_path)
    else:
        raise Exception(f"Unable to access: {autograder_materials_repo}")
    return storage_path


def remove_notebook():
    files = glob.glob('/tmp/*')
    for f in files:
        if not os.path.isdir(f):
            try:
                os.remove(f)
            except Exception:
                pass


async def grade_assignment(submission,
                           args,
                           sops_path=None, secrets_file=None, save_path=None):
    """
    This function spins up a docker instance using otter, grades the submission
    and returns the grade

    :param submission: the path to the file you want to grade
    :param args: json containing metadata from notebook
        - course: used as key to secrets file and course config
        - section: key to course config
        - assignment: key to assignment name
    :param sops_path: [OPTIONAL] used to execute pytests
    :param secrets_file: [OPTIONAL] used to execute pytests
    :param save_path: [OPTIONAL] used to execute pytests
    :return: grade
    :rtype: float
    """
    try:
        solutions_base_path = None
        if save_path is None:
            save_path = "/opt"
        if secrets_file is None:
            secrets_file = os.path.join(os.path.dirname(__file__), "secrets/gh_key.yaml")
        solutions_base_path = download_autograder_materials(args["course"], sops_path, secrets_file, save_path=save_path)
        course_config = util.get_course_config(solutions_base_path)
        print("===========")
        print(course_config)
        print("----------")
        autograder_subpath = course_config[args["course"]][args["section"]]["subpath_to_zips"]

        solutions_path = f'{solutions_base_path}/{autograder_subpath}/{args["assignment"]}/autograder.zip'

        command = [
            'otter', 'grade',
            '-a', solutions_path,
            '-p', submission
        ]
        process = await asyncio.create_subprocess_exec(
            *command,
            stdin=asyncio.subprocess.PIPE,
            stdout=asyncio.subprocess.PIPE,
            stderr=asyncio.subprocess.PIPE
        )

        # this is waiting for communication back from the process
        # some images are quite big and take some time to build the first
        # time through - like 20 min for otter-grader
        async with async_timeout.timeout(2000):
            stdout, stderr = await process.communicate()

        for line in stderr.decode('utf-8').split('\n'):
            if line.strip() == '':
                # Ignore empty lines
                continue
            if 'Killed' in line:
                # Our container was killed, so let's just skip this one
                raise Exception(f"Container was killed -- nothing will work: {submission}")

        grade = stdout.decode("utf-8").strip()
        if grade is None or grade == '':
            cmd = ' '.join(command)
            raise Exception(f"Unable to determine grade coming from otter on: {submission} using this commnad: {cmd}")

        return float(grade)
    except asyncio.TimeoutError:
        raise Exception(f'Grading timed out for {submission}')
    except Exception as e:
        raise e
    finally:
        remove_notebook()



import pytest
import otter_service.access_sops_keys as ask
import otter_service.grade_assignment as ga
import os
import shutil


@pytest.fixture()
def configure():
    print("Starting tests")
    yield "resource"
    print("Removing Tree")
    if os.path.isdir("./materials-x22-private"):
        shutil.rmtree("./materials-x22-private")


def test_download_autograder_materials(configure):
    key_test = "tests/test_files/gh_key.yaml"
    sops_path = "sops"
    ga.download_autograder_materials("8x", sops_path, key_test)
    assert os.path.isdir("./materials-x22-private")


@pytest.mark.asyncio
async def test_grade_assignment(configure):
    secrets_file = os.path.join(os.path.dirname(__file__), "test_files/gh_key.yaml")
    args = {
        "course": "8x",
        "section": "1",
        "assignment": "lab01",
        "autograder_materials_repo": "github.com/data-8/materials-x22-private"
    }
    grade = await ga.grade_assignment("tests/test_files/lab01.ipynb",
                                      args,
                                      sops_path="sops",
                                      secrets_file=secrets_file,
                                      save_path=".")
    assert 1.0 == grade
